@@ -1011,7 +1011,7 @@ resolve_types (char *buf, char *raw, struct line_t *lineInfo, int n, int last,
 
 static int is_ansi (const char *buf)
 {
-  while (*buf && (isdigit(*buf) || *buf == ';'))
+  while (*buf && (isdigit((unsigned char) *buf) || *buf == ';'))
     buf++;
   return (*buf == 'm');
 }
@@ -1179,7 +1179,7 @@ fill_buffer (FILE *f, LOFF_T *last_pos, LOFF_T offset, unsigned char **buf,
   if (*buf_ready == 0)
   {
     if (offset != *last_pos)
-      fseeko (f, offset, 0);
+      fseeko (f, offset, SEEK_SET);
 
     if ((*buf = (unsigned char *) mutt_read_line ((char *) *buf, blen, f,
                                                   NULL, MUTT_EOL)) == NULL)
@@ -1337,7 +1337,7 @@ static int format_line (struct line_t **lineInfo, int n, unsigned char *buf,
 
     if (Charset_is_utf8)
     {
-      if (wc == 0x200B || wc == 0xFEFF)
+      if (wc == 0x200B || wc == 0x200C || wc == 0xFEFF)
       {
 	dprint (3, (debugfile, "skip zero-width character U+%04X\n", (unsigned short)wc));
 	continue;
@@ -2161,6 +2161,44 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
   {
     mutt_curs_set (0);
 
+#if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
+    if (SigWinch)
+    {
+      do
+      {
+        SigWinch = 0;
+        mutt_resize_screen ();
+      }
+      while (SigWinch);
+
+      clearok(stdscr,TRUE);/*force complete redraw*/
+
+      if (flags & MUTT_PAGER_RETWINCH)
+      {
+        /* A SigWinch could occur while in the RETWINCH handler, so
+         * just keep the previously stored values */
+        if (!Resize)
+        {
+          /* Store current position. */
+          rd.lines = -1;
+          for (i = 0; i <= rd.topline; i++)
+            if (!rd.lineInfo[i].continuation)
+              rd.lines++;
+
+          Resize = safe_malloc (sizeof (struct resize));
+
+          Resize->line = rd.lines;
+          Resize->SearchCompiled = rd.SearchCompiled;
+          Resize->SearchBack = rd.SearchBack;
+        }
+
+	ch = -1;
+	rc = OP_REFORMAT_WINCH;
+        continue;
+      }
+    }
+#endif
+
     pager_menu_redraw (pager_menu);
 
     if (option(OPTBRAILLEFRIENDLY))
@@ -2188,40 +2226,6 @@ mutt_pager (const char *banner, const char *fname, int flags, pager_t *extra)
     else
       OldHdr = NULL;
 
-#if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
-    if (SigWinch)
-    {
-      SigWinch = 0;
-      mutt_resize_screen ();
-      clearok(stdscr,TRUE);/*force complete redraw*/
-
-      if (flags & MUTT_PAGER_RETWINCH)
-      {
-        /* Store current position. */
-        rd.lines = -1;
-        for (i = 0; i <= rd.topline; i++)
-          if (!rd.lineInfo[i].continuation)
-            rd.lines++;
-
-	Resize = safe_malloc (sizeof (struct resize));
-
-	Resize->line = rd.lines;
-	Resize->SearchCompiled = rd.SearchCompiled;
-	Resize->SearchBack = rd.SearchBack;
-
-	ch = -1;
-	rc = OP_REFORMAT_WINCH;
-      }
-      else
-      {
-        /* note: mutt_resize_screen() -> mutt_reflow_windows() sets
-         * REDRAW_FULL and REDRAW_FLOW */
-	ch = 0;
-      }
-      continue;
-    }
-#endif
-
     ch = km_dokey (MENU_PAGER);
     if (ch >= 0)
       mutt_clear_error ();
@@ -2545,7 +2549,6 @@ search_next:
 	{
 	  InHelp = 1;
 	  mutt_help (MENU_PAGER);
-	  pager_menu->redraw = REDRAW_FULL;
 	  InHelp = 0;
 	}
 	else
@@ -2554,7 +2557,6 @@ search_next:
 
       case OP_ERROR_HISTORY:
         mutt_error_history_display ();
-        pager_menu->redraw = REDRAW_FULL;
         break;
 
       case OP_PAGER_HIDE_QUOTED:
@@ -2573,33 +2575,65 @@ search_next:
 	{
 	  int dretval = 0;
 	  int new_topline = rd.topline;
+	  int num_quoted = 0;
 
-	  while ((new_topline < rd.lastLine ||
-		  (0 == (dretval = display_line (rd.fp, &rd.last_pos, &rd.lineInfo,
-			 new_topline, &rd.lastLine, &rd.maxLine, MUTT_TYPES | (flags & MUTT_PAGER_NOWRAP),
-                         &rd.QuoteList, &rd.q_level, &rd.force_redraw, &rd.SearchRE, rd.pager_window))))
-		 && rd.lineInfo[new_topline].type != MT_COLOR_QUOTED)
-	    new_topline++;
+          if (PagerSkipQuotedContext < 0)
+            PagerSkipQuotedContext = 0;
 
-	  if (dretval < 0)
-	  {
-	    mutt_error _("No more quoted text.");
-	    break;
+          /* Skip past previous "context" quoted lines */
+          if (PagerSkipQuotedContext > 0)
+          {
+	    while ((new_topline < rd.lastLine ||
+		    (0 == (dretval = display_line (rd.fp, &rd.last_pos, &rd.lineInfo,
+			   new_topline, &rd.lastLine, &rd.maxLine, MUTT_TYPES | (flags & MUTT_PAGER_NOWRAP),
+			   &rd.QuoteList, &rd.q_level, &rd.force_redraw, &rd.SearchRE, rd.pager_window))))
+		   && rd.lineInfo[new_topline].type == MT_COLOR_QUOTED)
+	    {
+	      new_topline++;
+	      num_quoted++;
+	    }
+
+	    if (dretval < 0)
+	    {
+	      mutt_error _("No more unquoted text after quoted text.");
+	      break;
+	    }
+          }
+
+          if (num_quoted <= PagerSkipQuotedContext)
+          {
+	    num_quoted = 0;
+	    while ((new_topline < rd.lastLine ||
+		    (0 == (dretval = display_line (rd.fp, &rd.last_pos, &rd.lineInfo,
+			   new_topline, &rd.lastLine, &rd.maxLine, MUTT_TYPES | (flags & MUTT_PAGER_NOWRAP),
+			   &rd.QuoteList, &rd.q_level, &rd.force_redraw, &rd.SearchRE, rd.pager_window))))
+		   && rd.lineInfo[new_topline].type != MT_COLOR_QUOTED)
+	      new_topline++;
+
+	    if (dretval < 0)
+	    {
+	      mutt_error _("No more quoted text.");
+	      break;
+	    }
+
+	    while ((new_topline < rd.lastLine ||
+		    (0 == (dretval = display_line (rd.fp, &rd.last_pos, &rd.lineInfo,
+			   new_topline, &rd.lastLine, &rd.maxLine, MUTT_TYPES | (flags & MUTT_PAGER_NOWRAP),
+			   &rd.QuoteList, &rd.q_level, &rd.force_redraw, &rd.SearchRE, rd.pager_window))))
+		   && rd.lineInfo[new_topline].type == MT_COLOR_QUOTED)
+	    {
+	      new_topline++;
+	      num_quoted++;
+	    }
+
+	    if (dretval < 0)
+	    {
+	      mutt_error _("No more unquoted text after quoted text.");
+	      break;
+	    }
 	  }
 
-	  while ((new_topline < rd.lastLine ||
-		  (0 == (dretval = display_line (rd.fp, &rd.last_pos, &rd.lineInfo,
-			 new_topline, &rd.lastLine, &rd.maxLine, MUTT_TYPES | (flags & MUTT_PAGER_NOWRAP),
-                         &rd.QuoteList, &rd.q_level, &rd.force_redraw, &rd.SearchRE, rd.pager_window))))
-		 && rd.lineInfo[new_topline].type == MT_COLOR_QUOTED)
-	    new_topline++;
-
-	  if (dretval < 0)
-	  {
-	    mutt_error _("No more unquoted text after quoted text.");
-	    break;
-	  }
-	  rd.topline = new_topline;
+	  rd.topline = new_topline - MIN (PagerSkipQuotedContext, num_quoted);
 	}
 	break;
 
@@ -2689,7 +2723,6 @@ search_next:
 			      extra->actx, extra->bdy);
         else
 	  mutt_resend_message (NULL, extra->ctx, extra->hdr);
-        pager_menu->redraw = REDRAW_FULL;
         break;
 
       case OP_CHECK_TRADITIONAL:
@@ -2712,7 +2745,6 @@ search_next:
 	else
 	  mutt_send_message (SENDTOSENDER | SENDBACKGROUNDEDIT,
                              NULL, NULL, extra->ctx, extra->hdr);
-	pager_menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_CREATE_ALIAS:
@@ -2862,7 +2894,6 @@ search_next:
         CHECK_ATTACH;
 	mutt_send_message (SENDBACKGROUNDEDIT | SENDCHECKPOSTPONED, NULL, NULL,
                            extra->ctx, NULL);
-	pager_menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_REPLY:
@@ -2885,7 +2916,6 @@ search_next:
 			     extra->bdy, replyflags);
 	else
 	  mutt_send_message (replyflags, NULL, NULL, extra->ctx, extra->hdr);
-	pager_menu->redraw = REDRAW_FULL;
 	break;
       }
 
@@ -2899,7 +2929,6 @@ search_next:
         CHECK_ATTACH;
 	mutt_send_message (SENDPOSTPONED | SENDBACKGROUNDEDIT,
                            NULL, NULL, extra->ctx, NULL);
-	pager_menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_FORWARD_MESSAGE:
@@ -2911,7 +2940,6 @@ search_next:
         else
 	  mutt_send_message (SENDFORWARD | SENDBACKGROUNDEDIT,
                              NULL, NULL, extra->ctx, extra->hdr);
-	pager_menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_DECRYPT_SAVE:
@@ -3060,7 +3088,6 @@ search_next:
 	mutt_view_attachments (extra->hdr);
 	if (extra->hdr->attach_del)
 	  Context->changed = 1;
-	pager_menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_EDIT_LABEL:
@@ -3087,7 +3114,6 @@ search_next:
 	CHECK_MODE(IsHeader(extra));
         CHECK_ATTACH;
 	mutt_send_message (SENDKEY, NULL, NULL, extra->ctx, NULL);
-	pager_menu->redraw = REDRAW_FULL;
 	break;
 
 
